/* _______________________________________________________________________
	TrinityVideo.h - Video pipe constants for Trinity
	
	first created: 4/3/2001
	
	copyright  1999-2001 by David DeHaven - all rights reserved.
 _______________________________________________________________________*/

#ifndef _TrinityVideo_
#define _TrinityVideo_

	#include <Trinity/TrinityCommon.h>

#ifdef __cplusplus
extern "C" {
#endif

enum {
	kTRPipeTypeVideo = 'Vdeo',			/* IN or OUT */
	
	/* Setting Selectors				   GET/SET IN/OUT - allowed access modes */
	kTRVideoSource = 'Vsrc',			/* GET/SET IN/OUT - selects the source of the video signal (numeric, from 0) */
	kTRVideoSourceList = 'VSCs',		/* GET     IN/OUT - get a list of video sources available */
	
	kTRVideoMode = 'Vmod',				/* GET/SET IN/OUT - choose or get the currently selected video mode, by mode ID */
	kTRVideoModeList = 'VMDs',			/* GET     IN/OUT - get a list of video modes available */
	kTRVideoPreferredMode = 'VPrM',		/* GET     IN/OUT - the preferred mode based on current settings, typically yields best performance */
	
	kTRVideoFormat = 'Vfmt',			/* GET     IN/OUT - type of data being handled (jpeg, YUV, RGB, etc...) Use QuickTime selectors */
	kTRVideoStandard = 'Vstd',			/* GET     IN/OUT - The current video standard */
	kTRVideoSize = 'Vsiz',				/* GET     IN/OUT - current video image size */
	kTRVideoAspectRatio = 'Vasp',		/* GET     IN/OUT - aspect ratio of the video stream */
	
	kTRVideoTimeScale = 'Vtsc',			/* GET     IN     - get the number of units/second in each time stamp, for instance a value of 60 means 60 units per second */
	
	kTRVideoStatus = 'Vsta',			/* GET     IN     - if a video signal is present */
	
	kTRVideoHue = 'Vhue',				/* GET/SET IN/OUT - set image hue (0-65535) */
	kTRVideoBrightness = 'Vbrt',		/* GET/SET IN/OUT - set image brightness (0-65535) */
	kTRVideoContrast = 'Vcon',			/* GET/SET IN/OUT - set image contrast (0-65535) */
	kTRVideoSaturation = 'Vsat',		/* GET/SET IN/OUT - set image saturation (0-65535) */
	kTRVideoSharpness = 'Vshp',			/* GET/SET IN/OUT - set image sharpness (0-65535) */
	
	kTRVideoImageDescription = 'VDsc',	/* GET     IN     - image description data for decompression with QuickTime(tm), data returned is an ImageDescriptionPtr */

	kTRVideoFrameSize = 'Vfsz',			/* GET     IN     - returns the maximum size of each frame in bytes, given the format, image size and other settings */
	kTRVideoFrameRate = 'Vfrt',			/*     SET IN/OUT - frame rate control (or feedback) for devices that support it */
	kTRVideoBitRate = 'Vbtr',			/*     SET IN/OUT - data bitrate control for devices that can control bandwidth usage */
};

/*
	Video input list
	
	Data structure returned as a result of calling TRGetPipeSetting with the kTRVideoSourceList selector.
	
	When you want to select a source, you call TRSetPipeSetting with the kTRVideoSource selector and the
	data set to the index (zero based) of the desired source.
	
	You should poll the driver to get the current source rather than force it to some index when you first
	access the device. Other clients may be using the device, changing the source while in use may not be
	what the user desires.
	
	You are responsible for deallocating using free().
	
	Note that when you change video sources, the mode list could also change as well as the current mode.
	The default mode is dependent on the driver.
*/
struct TRVideoSourceInfo {
	unsigned long type;		// what kind of video source
	char name[32];			// a descriptive name for this video source
};
typedef struct TRVideoSourceInfo TRVideoSourceInfo;

struct TRVideoSourceList {
	long count;
	TRVideoSourceInfo list[1];
};
typedef struct TRVideoSourceList TRVideoSourceList;

/*
	Video mode list
	
	The video mode list is returned as an array of these structures. You are responsible for 
	disposing of the list when you are finished with it.
	
	After you determine or set the video source, get a list of available video modes using TRGetPipeSetting
	and the kTRVideoMode selector. Changing the input could change the mode list, so you must remember to
	update the mode list and verify the current mode. The driver is responsible for selecting a mode when
	the source is changed, but it may or may not be similar to the mode that was selected in the previous
	source.
*/
	// set the version field to this value when passing mode information to the client
#define kTRVideoModeCurrentVersion 0x00010000

struct TRVideoMode {
	unsigned long version;	// version of this structure
	unsigned long modeID;	// Argument for kTRVideoCurrentMode when setting mode
	char modeName[32];		// Human readable name for this mode (could be NULL string)
	
	unsigned long dataFormat;	// How the video data is encoded (ie: mjpa, mpeg, YUV, etc...)
	
	unsigned long videoStandard;	// What video standard is used (NTSC, PAL, SECAM, HD)
	unsigned long videoSize;		// Size of the image
	unsigned long videoAspectRatio;	// The correct aspect ratio (may not match width and height)
};
typedef struct TRVideoMode TRVideoMode;

/* video pipe constants */

enum {
	/* video input/output source types */
	kTRVideoSourceType_Composite = 0,
	kTRVideoSourceType_SVideo,
	kTRVideoSourceType_Tuner,
	kTRVideoSourceType_VGA,				/* outputs to a VGA monitor */
	kTRVideoSourceType_RGB,				/* separate RGB inputs (various types) */
	kTRVideoSourceType_SDI,				/* for digital TV's */
	kTRVideoSourceType_1394,			/* from a 1394 camera or video in device */
	
	/* video standards supported... these are more for color information decoding
	   since actual sizes and aspect ratios are given in the video mode list
	 */
	kTRVideoStd_NTSC = 'NTSC',			/* 4:3 525 line 60 Hz */
	kTRVideoStd_PAL = 'PAL ',			/* 4:3 625 line 50 Hz */
	kTRVideoStd_SECAM = 'SECM',
	kTRVideoStd_HD = ' HD ',			/* any of the high def standards */
	
	/* Video aspect ratios */
		/* aspect ratio is 32 bits, the high word is the enumerator, low word the denominator
		   for instance: 4:3 is 0x00040003 */
	kTRAspectRatio_Square = 0x00010001,
	kTRAspectRatio_4to3 = 0x00040003,
	kTRAspectRatio_16to9 = 0x00100009,	/* others? these are all the ATSC defined ratios */
	
	/* video frame rates, actual decimal rate times 100 */
	/* other values may be used, these are simply the most common ones used */
	kTRFrameRate_10 = 1000,
	kTRFrameRate_15 = 1500,
	kTRFrameRate_2397 = 2397,	/* 23.976 fps */
	kTRFrameRate_24 = 2400,		/* 24 fps For ATSC digital standards, film rate compatibility */
	kTRFrameRate_2997 = 2997,	/* 29.97 fps */
	kTRFrameRate_30 = 3000,
	kTRFrameRate_5994 = 5994,	/* 54.94 fps */
	kTRFrameRate_60 = 6000,
	
	/* standard video sizes */
		/* 32 bits total: High 16 bits = width, Low 16 bits = height (Big endian)
		   The driver can define whatever sizes it wants, these are just convenient */
	kTRVideoSize_NTSC_Full = 0x028001E0,	/* 640x480 square pixel */
	kTRVideoSize_NTSC_CIF = (kTRVideoSize_NTSC_Full>>1),
	kTRVideoSize_NTSC_QCIF = (kTRVideoSize_NTSC_Full>>2),
	
	kTRVideoSize_PAL_Full = 0x02D00240,		/* 720x576 square pixel */
	kTRVideoSize_PAL_CIF = (kTRVideoSize_PAL_Full>>1),
	kTRVideoSize_PAL_QCIF = (kTRVideoSize_PAL_Full>>2),
	
	kTRVideoSize_SECAM_Full = kTRVideoSize_PAL_Full,
	kTRVideoSize_SECAM_CIF = (kTRVideoSize_SECAM_Full>>1),
	kTRVideoSize_SECAM_QCIF = (kTRVideoSize_SECAM_Full>>2),
	
	kTRVideoSize_NTSC_CCIR_Full = 0x02D001E0,	/* 720x480 CCIR */
	kTRVideoSize_NTSC_CCIR_CIF = (kTRVideoSize_NTSC_CCIR_Full>>1),
	kTRVideoSize_NTSC_CCIR_QCIF = (kTRVideoSize_NTSC_CCIR_Full>>2),
	
	kTRVideoSize_PAL_CCIR_Full = 0x03000240,		/* 768x576 */
	kTRVideoSize_PAL_CCIR_CIF = (kTRVideoSize_PAL_CCIR_Full>>1),
	kTRVideoSize_PAL_CCIR_QCIF = (kTRVideoSize_PAL_CCIR_Full>>2),
	
	kTRVideoSize_HD_1920_1080 = 0x07800438,	/* HDTV resolutions */
	kTRVideoSize_HD_1280_720 = 0x050002D0,
	kTRVideoSize_SD_640_480 = 0x028001E0,	/* SDTV resolutions */
	kTRVideoSize_SD_704_480 = 0x02C001E0
};

enum {
		/* Video pipe errors */
	kTRVideoInvalidSourceErr = 1000,
	kTRVideoInvalidModeErr,
	kTRVideoSettingOutOfRangeErr,
	kTRVideoFrameRateNotAdjustableErr
};

#ifdef __cplusplus
}
#endif

#endif /* _TrinityVideo_ */

/*______________________________________________________________________*/
